@@ -13,7 +13,9 @@ following sub-packages:
    fsl.data
    fsl.utils
    fsl.scripts
+   fsl.transform
    fsl.version
+   fsl.wrappers
 
 .. note:: The ``fsl`` namespace is a ``pkgutil``-style *namespace package* -
           it can be used across different projects - see
@@ -21,4 +23,4 @@ following sub-packages:
           for details.
 """
 
-__path__ = __import__('pkgutil').extend_path(__path__, __name__)
+__path__ = __import__('pkgutil').extend_path(__path__, __name__)  # noqa
